@@ -30,6 +30,8 @@
 #include <pwd.h>
 #include <unistd.h>
 #include <errno.h>
+#include <unistd.h>
+#include <sys/types.h>
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -46,7 +48,11 @@
 
 #define PATH_PASSWD "/etc/passwd"
 #define PATH_SHADOW "/etc/shadow"
-#define MINIMAL_UID 500
+#define PATH_LOGIN_DEFS "/etc/login.defs"
+
+#ifndef FALLBACK_MINIMAL_UID
+#define FALLBACK_MINIMAL_UID 500
+#endif
 
 #define USERDIR LOCALSTATEDIR "/lib/AccountsService/users"
 
@@ -82,6 +88,7 @@ enum {
 enum {
         USER_ADDED,
         USER_REMOVED,
+        CK_HISTORY_LOADED,
         LAST_SIGNAL
 };
 
@@ -93,6 +100,7 @@ struct DaemonPrivate {
 
         GHashTable *users;
         GHashTable *exclusions;
+        uid_t minimal_uid;
 
         User *autologin;
 
@@ -197,6 +205,16 @@ daemon_class_init (DaemonClass *klass)
                                               1,
                                               DBUS_TYPE_G_OBJECT_PATH);
 
+        signals[CK_HISTORY_LOADED] = g_signal_new ("ck-history-loaded",
+                                              G_OBJECT_CLASS_TYPE (klass),
+                                              G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+                                              0,
+                                              NULL,
+                                              NULL,
+                                              g_cclosure_marshal_VOID__VOID,
+                                              G_TYPE_NONE,
+                                              0);
+
         dbus_g_object_type_install_info (TYPE_DAEMON,
                                          &dbus_glib_daemon_object_info);
 
@@ -213,10 +231,10 @@ daemon_class_init (DaemonClass *klass)
                                                               G_PARAM_READABLE));
 }
 
-static gboolean
-user_is_excluded (Daemon *daemon, const gchar *username, uid_t uid)
+gboolean
+daemon_local_user_is_excluded (Daemon *daemon, const gchar *username, uid_t uid)
 {
-        if (uid < MINIMAL_UID) {
+        if (uid < daemon->priv->minimal_uid) {
                 return TRUE;
         }
         if (g_hash_table_lookup (daemon->priv->exclusions, username)) {
@@ -337,8 +355,7 @@ process_ck_history_line (Daemon      *daemon,
                 return;
         }
 
-        /* pass MINIMAL_UID to just check the name here */
-        if (user_is_excluded (daemon, username, MINIMAL_UID)) {
+        if (daemon_local_user_is_excluded (daemon, username, daemon->priv->minimal_uid)) {
                 g_debug ("excluding user '%s'", username);
                 g_free (username);
                 return;
@@ -387,6 +404,7 @@ ck_history_watch (GIOChannel   *source,
 
         if (done) {
                 daemon->priv->ck_history_id = 0;
+                g_signal_emit (daemon, signals[CK_HISTORY_LOADED], 0);
                 return FALSE;
         }
 
@@ -487,7 +505,7 @@ reload_passwd (Daemon *daemon)
 
         for (pwent = fgetpwent (fp); pwent != NULL; pwent = fgetpwent (fp)) {
                 /* Skip users below MINIMAL_UID... */
-                if (user_is_excluded (daemon, pwent->pw_name, pwent->pw_uid)) {
+                if (daemon_local_user_is_excluded (daemon, pwent->pw_name, pwent->pw_uid)) {
                         g_debug ("skipping user: %s", pwent->pw_name);
                         continue;
                 }
@@ -601,7 +619,7 @@ reload_autologin_timeout (Daemon *daemon)
         daemon->priv->autologin_id = 0;
 
         if (!load_autologin (daemon, &name, &enabled, &error)) {
-                g_warning ("failed to load gdms custom.conf: %s", error->message);
+                g_debug ("failed to load gdms custom.conf: %s", error->message);
                 g_error_free (error);
                 g_free (name);
 
@@ -658,6 +676,59 @@ on_passwd_monitor_changed (GFileMonitor      *monitor,
         reload_users (daemon);
 }
 
+static uid_t
+get_minimal_uid (void)
+{
+        GError *error;
+        char *contents;
+        gboolean contents_loaded;
+        const char *uid_min_string, *start_of_uid_string;
+        char *end;
+        uid_t uid = FALLBACK_MINIMAL_UID;
+        guint64 uid_as_number;
+
+        error = NULL;
+        contents = NULL;
+        contents_loaded = g_file_get_contents (PATH_LOGIN_DEFS, &contents, NULL, &error);
+        if (!contents_loaded) {
+                g_debug ("unable to read " PATH_LOGIN_DEFS ": %s", error->message);
+                g_error_free (error);
+                goto out;
+        }
+
+        uid_min_string = strstr (contents, "UID_MIN");
+
+        if (uid_min_string == NULL ||
+            (uid_min_string != contents && uid_min_string[-1] != '\n')) {
+                g_debug (PATH_LOGIN_DEFS " does not have a UID_MIN field");
+                goto out;
+        }
+
+        start_of_uid_string = uid_min_string + strlen ("UID_MIN");
+
+        if (start_of_uid_string == '\0') {
+                g_debug (PATH_LOGIN_DEFS " contains UID_MIN key with no value");
+                goto out;
+        }
+
+
+        uid_as_number = g_ascii_strtoll (start_of_uid_string, &end, 10);
+        if (!g_ascii_isspace (*end) && *end != '\0') {
+                g_debug (PATH_LOGIN_DEFS " contains non-numerical value for UID_MIN");
+                goto out;
+        }
+
+        if (uid_as_number < 0 || ((uid_t) uid_as_number) != uid_as_number) {
+                g_debug (PATH_LOGIN_DEFS " contains out-of-range value for UID_MIN");
+                goto out;
+        }
+
+        uid = (uid_t) uid_as_number;
+out:
+        g_free (contents);
+        return uid;
+}
+
 static void
 daemon_init (Daemon *daemon)
 {
@@ -667,6 +738,7 @@ daemon_init (Daemon *daemon)
 
         daemon->priv = DAEMON_GET_PRIVATE (daemon);
 
+        daemon->priv->minimal_uid = get_minimal_uid ();
         daemon->priv->exclusions = g_hash_table_new_full (g_str_hash,
                                                           g_str_equal,
                                                           g_free,
@@ -854,7 +926,7 @@ daemon_local_find_user_by_id (Daemon *daemon,
 
         pwent = getpwuid (uid);
         if (pwent == NULL) {
-                g_warning ("unable to lookup uid %d", (int)uid);
+                g_debug ("unable to lookup uid %d", (int)uid);
                 return NULL;
         }
 
@@ -875,7 +947,7 @@ daemon_local_find_user_by_name (Daemon      *daemon,
 
         pwent = getpwnam (name);
         if (pwent == NULL) {
-                g_warning ("unable to lookup name %s", name);
+                g_debug ("unable to lookup name %s", name);
                 return NULL;
         }
 
@@ -969,7 +1041,7 @@ finish_list_cached_users (gpointer user_data)
         g_hash_table_iter_init (&iter, data->daemon->priv->users);
         while (g_hash_table_iter_next (&iter, (gpointer *)&name, (gpointer *)&user)) {
                 uid = user_local_get_uid (user);
-                if (!user_is_excluded (data->daemon, name, uid)) {
+                if (!daemon_local_user_is_excluded (data->daemon, name, uid)) {
                         g_debug ("user %s %ld not excluded\n", name, uid);
                         g_ptr_array_add (object_paths, g_strdup (user_local_get_object_path (user)));
                 }
@@ -985,6 +1057,18 @@ finish_list_cached_users (gpointer user_data)
         return FALSE;
 }
 
+static void
+on_ck_history_loaded (Daemon       *daemon,
+                      ListUserData *data)
+{
+        /* ck-history loaded, so finish pending ListCachedUsers call */
+        g_idle_add (finish_list_cached_users, data);
+
+        g_signal_handlers_disconnect_by_func (daemon,
+                                              on_ck_history_loaded,
+                                              data);
+}
+
 gboolean
 daemon_list_cached_users (Daemon                *daemon,
                           DBusGMethodInvocation *context)
@@ -993,8 +1077,13 @@ daemon_list_cached_users (Daemon                *daemon,
 
         data = list_user_data_new (daemon, context);
 
-        if (daemon->priv->reload_id > 0) {
-                /* reload in progress, wait */
+        if (daemon->priv->ck_history_id > 0) {
+                /* loading ck-history, wait for it */
+                g_signal_connect (daemon, "ck-history-loaded",
+                                  G_CALLBACK (on_ck_history_loaded),
+                                  data);
+        } else if (daemon->priv->reload_id > 0) {
+                /* reload in progress, wait a bit */
                 g_idle_add (finish_list_cached_users, data);
         }
         else {
@@ -1046,20 +1135,18 @@ daemon_create_user_authorized_cb (Daemon                *daemon,
         argv[3] = cd->real_name;
         if (cd->account_type == ACCOUNT_TYPE_ADMINISTRATOR) {
                 argv[4] = "-G";
-                argv[5] = "desktop_admin_r";
+                argv[5] = "wheel";
                 argv[6] = cd->user_name;
                 argv[7] = NULL;
         }
         else if (cd->account_type == ACCOUNT_TYPE_STANDARD) {
-                argv[4] = "-G";
-                argv[5] = "desktop_user_r";
-                argv[6] = cd->user_name;
-                argv[7] = NULL;
-        }
-        else {
                 argv[4] = cd->user_name;
                 argv[5] = NULL;
         }
+        else {
+                throw_error (context, ERROR_FAILED, "Don't know how to add user of type %d", cd->account_type);
+                return;
+        }
 
         error = NULL;
         if (!spawn_with_login_uid (context, argv, &error)) {
@@ -1313,7 +1400,7 @@ load_autologin (Daemon      *daemon,
                 g_free (string);
                 return FALSE;
         }
-        if (g_strcmp0 (string, "True") == 0) {
+        if (string != NULL && (g_ascii_strcasecmp (string, "true") == 0 || strcmp (string, "1") == 0)) {
                 *enabled = TRUE;
         }
         else {
