@@ -140,6 +140,7 @@ struct ActUserManagerPrivate
 
         GSList                *new_sessions;
         GSList                *new_users;
+        GSList                *new_users_inhibiting_load;
         GSList                *fetch_user_requests;
 
         GSList                *exclude_usernames;
@@ -186,8 +187,8 @@ static void     set_is_loaded (ActUserManager *manager, gboolean is_loaded);
 static void     on_new_user_loaded (ActUser        *user,
                                     GParamSpec     *pspec,
                                     ActUserManager *manager);
-static void     give_up_and_fetch_user_locally (ActUserManager                 *manager,
-                                                ActUserManagerFetchUserRequest *request);
+static void     give_up (ActUserManager                 *manager,
+                         ActUserManagerFetchUserRequest *request);
 static void     fetch_user_incrementally       (ActUserManagerFetchUserRequest *request);
 
 static void     maybe_set_is_loaded            (ActUserManager *manager);
@@ -533,7 +534,8 @@ on_user_changed (ActUser        *user,
                  ActUserManager *manager)
 {
         if (manager->priv->is_loaded) {
-                g_debug ("ActUserManager: user changed");
+                g_debug ("ActUserManager: user %s changed",
+                         act_user_get_user_name (user));
                 g_signal_emit (manager, signals[USER_CHANGED], 0, user);
         }
 }
@@ -570,8 +572,9 @@ on_get_seat_id_finished (DBusGProxy     *proxy,
                         g_debug ("Failed to identify the seat of the "
                                  "current session");
                 }
+
+                g_debug ("ActUserManager: GetSeatId call failed, so unloading seat");
                 unload_seat (manager);
-                maybe_set_is_loaded (manager);
                 return;
         }
 
@@ -692,6 +695,7 @@ add_user (ActUserManager *manager,
 {
         const char *object_path;
 
+        g_debug ("ActUserManager: tracking user '%s'", act_user_get_user_name (user));
         g_hash_table_insert (manager->priv->users_by_name,
                              g_strdup (act_user_get_user_name (user)),
                              g_object_ref (user));
@@ -713,7 +717,10 @@ add_user (ActUserManager *manager,
                           manager);
 
         if (manager->priv->is_loaded) {
+                g_debug ("ActUserManager: loaded, so emitting user-added signal");
                 g_signal_emit (manager, signals[USER_ADDED], 0, user);
+        } else {
+                g_debug ("ActUserManager: not yet loaded, so not emitting user-added signal");
         }
 
         if (g_hash_table_size (manager->priv->users_by_name) > 1) {
@@ -725,6 +732,10 @@ static void
 remove_user (ActUserManager *manager,
              ActUser        *user)
 {
+        g_debug ("ActUserManager: no longer tracking user '%s' (with object path %s)",
+                 act_user_get_user_name (user),
+                 act_user_get_object_path (user));
+
         g_object_ref (user);
 
         g_signal_handlers_disconnect_by_func (user, on_user_changed, manager);
@@ -732,10 +743,16 @@ remove_user (ActUserManager *manager,
         if (act_user_get_object_path (user) != NULL) {
                 g_hash_table_remove (manager->priv->users_by_object_path, act_user_get_object_path (user));
         }
-        g_hash_table_remove (manager->priv->users_by_name, act_user_get_user_name (user));
+        if (act_user_get_user_name (user) != NULL) {
+                g_hash_table_remove (manager->priv->users_by_name, act_user_get_user_name (user));
+
+        }
 
         if (manager->priv->is_loaded) {
+                g_debug ("ActUserManager: loaded, so emitting user-removed signal");
                 g_signal_emit (manager, signals[USER_REMOVED], 0, user);
+        } else {
+                g_debug ("ActUserManager: not yet loaded, so not emitting user-removed signal");
         }
 
         g_object_unref (user);
@@ -754,12 +771,16 @@ on_new_user_loaded (ActUser        *user,
         ActUser *old_user;
 
         if (!act_user_is_loaded (user)) {
+                g_debug ("ActUserManager: user '%s' loaded function called when not loaded",
+                         act_user_get_user_name (user));
                 return;
         }
-
         g_signal_handlers_disconnect_by_func (user, on_new_user_loaded, manager);
+
         manager->priv->new_users = g_slist_remove (manager->priv->new_users,
                                                    user);
+        manager->priv->new_users_inhibiting_load = g_slist_remove (manager->priv->new_users_inhibiting_load,
+                                                                   user);
 
         username = act_user_get_user_name (user);
 
@@ -777,13 +798,15 @@ on_new_user_loaded (ActUser        *user,
                                    (int) act_user_get_uid (user));
                 }
                 g_object_unref (user);
-                return;
+                goto out;
         }
 
+        g_debug ("ActUserManager: user '%s' is now loaded", username);
+
         if (username_in_exclude_list (manager, username)) {
                 g_debug ("ActUserManager: excluding user '%s'", username);
                 g_object_unref (user);
-                return;
+                goto out;
         }
 
         old_user = g_hash_table_lookup (manager->priv->users_by_name, username);
@@ -791,14 +814,20 @@ on_new_user_loaded (ActUser        *user,
         /* If username got added earlier by a different means, trump it now.
          */
         if (old_user != NULL) {
+                g_debug ("ActUserManager: user '%s' was already known, "
+                         "replacing with freshly loaded object", username);
                 remove_user (manager, old_user);
         }
 
         add_user (manager, user);
         g_object_unref (user);
 
-        if (manager->priv->new_users == NULL) {
-                set_is_loaded (manager, TRUE);
+out:
+        if (manager->priv->new_users_inhibiting_load == NULL) {
+                g_debug ("ActUserManager: no pending users, trying to set loaded property");
+                maybe_set_is_loaded (manager);
+        } else {
+                g_debug ("ActUserManager: not all users loaded yet");
         }
 }
 
@@ -811,8 +840,13 @@ add_new_user_for_object_path (const char     *object_path,
         user = g_hash_table_lookup (manager->priv->users_by_object_path, object_path); 
 
         if (user != NULL) {
+                g_debug ("ActUserManager: tracking existing user %s with object path %s",
+                         act_user_get_user_name (user), object_path);
                 return user;
         }
+
+        g_debug ("ActUserManager: tracking new user with object path %s", object_path);
+
         user = create_new_user (manager);
         _act_user_update_from_object_path (user, object_path);
 
@@ -826,6 +860,12 @@ on_new_user_in_accounts_service (DBusGProxy *proxy,
 {
         ActUserManager *manager = ACT_USER_MANAGER (user_data);
 
+        if (!manager->priv->is_loaded) {
+                g_debug ("ActUserManager: ignoring new user in accounts service with object path %s since not loaded yet", object_path);
+                return;
+        }
+
+        g_debug ("ActUserManager: new user in accounts service with object path %s", object_path);
         add_new_user_for_object_path (object_path, manager);
 }
 
@@ -839,6 +879,13 @@ on_user_removed_in_accounts_service (DBusGProxy *proxy,
 
         user = g_hash_table_lookup (manager->priv->users_by_object_path, object_path);
 
+        if (user == NULL) {
+                g_debug ("ActUserManager: ignoring untracked user %s", object_path);
+                return;
+        } else {
+                g_debug ("ActUserManager: tracked user %s removed from accounts service", object_path);
+        }
+
         manager->priv->new_users = g_slist_remove (manager->priv->new_users, user);
 
         remove_user (manager, user);
@@ -876,7 +923,6 @@ on_get_current_session_finished (DBusGProxy     *proxy,
                         g_debug ("Failed to identify the current session");
                 }
                 unload_seat (manager);
-                maybe_set_is_loaded (manager);
                 return;
         }
 
@@ -1072,7 +1118,7 @@ on_find_user_by_name_finished (DBusGProxy                     *proxy,
                         g_debug ("ActUserManager: Failed to find user %s",
                                  request->username);
                 }
-                give_up_and_fetch_user_locally (manager, request);
+                give_up (manager, request);
                 return;
         }
 
@@ -1115,7 +1161,7 @@ find_user_in_accounts_service (ActUserManager                 *manager,
         return;
 
 failed:
-        give_up_and_fetch_user_locally (manager, request);
+        give_up (manager, request);
 }
 
 static void
@@ -1128,6 +1174,19 @@ set_is_loaded (ActUserManager *manager,
         }
 }
 
+static void
+add_new_inhibiting_user_for_object_path (const char     *object_path,
+                                         ActUserManager *manager)
+{
+        ActUser *user;
+
+        user = add_new_user_for_object_path (object_path, manager);
+
+        if (!manager->priv->is_loaded) {
+                manager->priv->new_users_inhibiting_load = g_slist_prepend (manager->priv->new_users_inhibiting_load, user);
+        }
+}
+
 static void
 on_list_cached_users_finished (DBusGProxy     *proxy,
                                DBusGProxyCall *call_id,
@@ -1152,8 +1211,18 @@ on_list_cached_users_finished (DBusGProxy     *proxy,
                 return;
         }
 
-        maybe_set_is_loaded (manager);
-        g_ptr_array_foreach (paths, (GFunc)add_new_user_for_object_path, manager);
+        /* We now have a batch of unloaded users that we know about. Once that initial
+         * batch is loaded up, we can mark the manager as loaded.
+         *
+         * (see on_new_user_loaded)
+         */
+        if (paths->len > 0) {
+                g_debug ("ActUserManager: ListCachedUsers finished, will set loaded property after list is fully loaded");
+                g_ptr_array_foreach (paths, (GFunc)add_new_inhibiting_user_for_object_path, manager);
+        } else {
+                g_debug ("ActUserManager: ListCachedUsers finished with empty list, maybe setting loaded property now");
+                maybe_set_is_loaded (manager);
+        }
 
         g_ptr_array_foreach (paths, (GFunc)g_free, NULL);
         g_ptr_array_free (paths, TRUE);
@@ -1482,6 +1551,9 @@ unload_seat (ActUserManager *manager)
 
         g_free (manager->priv->seat.session_id);
         manager->priv->seat.session_id = NULL;
+
+        g_debug ("ActUserManager: seat unloaded, so trying to set loaded property");
+        maybe_set_is_loaded (manager);
 }
 
 static void
@@ -1557,13 +1629,12 @@ free_fetch_user_request (ActUserManagerFetchUserRequest *request)
 }
 
 static void
-give_up_and_fetch_user_locally (ActUserManager                 *manager,
-                                ActUserManagerFetchUserRequest *request)
+give_up (ActUserManager                 *manager,
+         ActUserManagerFetchUserRequest *request)
 {
 
         g_debug ("ActUserManager: account service unavailable, "
-                 "fetching user %s locally",
-                 request->username);
+                 "giving up");
         request->state = ACT_USER_MANAGER_GET_USER_STATE_UNFETCHED;
 }
 
@@ -1576,6 +1647,9 @@ on_user_manager_maybe_ready_for_request (ActUserManager                 *manager
                 return;
         }
 
+        g_debug ("ActUserManager: user manager now loaded, proceeding with fetch user request for user '%s'",
+                 request->username);
+
         g_signal_handlers_disconnect_by_func (manager, on_user_manager_maybe_ready_for_request, request);
 
         request->state++;
@@ -1606,7 +1680,7 @@ fetch_user_incrementally (ActUserManagerFetchUserRequest *request)
 
         case ACT_USER_MANAGER_GET_USER_STATE_ASK_ACCOUNTS_SERVICE:
                 if (manager->priv->accounts_proxy == NULL) {
-                        give_up_and_fetch_user_locally (manager, request);
+                        give_up (manager, request);
                 } else {
                         find_user_in_accounts_service (manager, request);
                 }
@@ -1674,6 +1748,7 @@ act_user_manager_get_user (ActUserManager *manager,
 
         /* if we don't have it loaded try to load it now */
         if (user == NULL) {
+                g_debug ("ActUserManager: trying to track new user with username %s", username);
                 user = create_new_user (manager);
 
                 if (manager->priv->accounts_proxy != NULL) {
@@ -1719,22 +1794,34 @@ static void
 maybe_set_is_loaded (ActUserManager *manager)
 {
         if (manager->priv->is_loaded) {
+                g_debug ("ActUserManager: already loaded, so not setting loaded property");
                 return;
         }
 
         if (manager->priv->get_sessions_call != NULL) {
+                g_debug ("ActUserManager: GetSessions call pending, so not setting loaded property");
                 return;
         }
 
         if (manager->priv->listing_cached_users) {
+                g_debug ("ActUserManager: Listing cached users, so not setting loaded property");
+                return;
+        }
+
+        if (manager->priv->new_users_inhibiting_load != NULL) {
+                g_debug ("ActUserManager: Loading new users, so not setting loaded property");
                 return;
         }
 
         /* Don't set is_loaded yet unless the seat is already loaded
          * or failed to load.
          */
-        if (manager->priv->seat.state != ACT_USER_MANAGER_SEAT_STATE_LOADED
-            && manager->priv->seat.state != ACT_USER_MANAGER_SEAT_STATE_UNLOADED) {
+        if (manager->priv->seat.state == ACT_USER_MANAGER_SEAT_STATE_LOADED) {
+                g_debug ("ActUserManager: Seat loaded, so now setting loaded property");
+        } else if (manager->priv->seat.state == ACT_USER_MANAGER_SEAT_STATE_UNLOADED) {
+                g_debug ("ActUserManager: Seat wouldn't load, so giving up on it and setting loaded property");
+        } else {
+                g_debug ("ActUserManager: Seat still actively loading, so not setting loaded property");
                 return;
         }
 
@@ -1809,6 +1896,8 @@ on_get_sessions_finished (DBusGProxy     *proxy,
                                   (int) sessions->len);
         g_ptr_array_foreach (sessions, (GFunc) g_free, NULL);
         g_ptr_array_free (sessions, TRUE);
+
+        g_debug ("ActUserManager: GetSessions call finished, so trying to set loaded property");
         maybe_set_is_loaded (manager);
 }
 
@@ -1878,6 +1967,7 @@ load_seat_incrementally (ActUserManager *manager)
                 load_sessions (manager);
         }
 
+        g_debug ("ActUserManager: Seat loading sequence complete, so trying to set loaded property");
         maybe_set_is_loaded (manager);
 }
 
@@ -2131,6 +2221,8 @@ act_user_manager_finalize (GObject *object)
                          (GFunc) free_fetch_user_request, NULL);
         g_slist_free (manager->priv->fetch_user_requests);
 
+        g_slist_free (manager->priv->new_users_inhibiting_load);
+
         node = manager->priv->new_users;
         while (node != NULL) {
                 ActUser *user;
@@ -2200,3 +2292,44 @@ act_user_manager_get_default (void)
 
         return ACT_USER_MANAGER (user_manager_object);
 }
+
+
+gboolean
+act_user_manager_create_user (ActUserManager     *manager,
+                              const char         *username,
+                              const char         *fullname,
+                              ActUserAccountType  accounttype)
+{
+        GError *error;
+        gboolean res;
+        gchar *path;
+
+        g_debug ("ActUserManager: Creating user '%s', '%s', %d",
+                 username, fullname, accounttype);
+
+        g_assert (manager->priv->accounts_proxy != NULL);
+
+        error = NULL;
+        res = dbus_g_proxy_call (manager->priv->accounts_proxy,
+                                 "CreateUser",
+                                 &error,
+                                 G_TYPE_STRING, username,
+                                 G_TYPE_STRING, fullname,
+                                 G_TYPE_INT, accounttype,
+                                 G_TYPE_INVALID,
+                                 DBUS_TYPE_G_OBJECT_PATH, &path,
+                                 G_TYPE_INVALID);
+        if (! res) {
+                if (error != NULL) {
+                        g_warning ("Failed to create user: %s", error->message);
+                        g_error_free (error);
+                } else {
+                        g_warning ("Failed to create user");
+                }
+                goto out;
+        }
+out:
+        g_free (path);
+
+        return res;
+}
