@@ -55,6 +55,7 @@ enum {
         PROP_LOCATION,
         PROP_LOCKED,
         PROP_AUTOMATIC_LOGIN,
+        PROP_SYSTEM_ACCOUNT,
         PROP_LOGIN_FREQUENCY,
         PROP_ICON_FILE,
         PROP_LANGUAGE,
@@ -97,6 +98,7 @@ struct _ActUser {
         guint           is_loaded : 1;
         guint           locked : 1;
         guint           automatic_login : 1;
+        guint           system_account : 1;
 };
 
 struct _ActUserClass
@@ -244,6 +246,9 @@ act_user_get_property (GObject    *object,
         case PROP_AUTOMATIC_LOGIN:
                 g_value_set_boolean (value, user->automatic_login);
                 break;
+        case PROP_SYSTEM_ACCOUNT:
+                g_value_set_boolean (value, user->system_account);
+                break;
         case PROP_IS_LOADED:
                 g_value_set_boolean (value, user->is_loaded);
                 break;
@@ -279,7 +284,7 @@ act_user_class_init (ActUserClass *class)
                                                            "Account Type",
                                                            "The account type for this user.",
                                                            ACT_USER_ACCOUNT_TYPE_STANDARD,
-                                                           ACT_USER_ACCOUNT_TYPE_SUPERVISED,
+                                                           ACT_USER_ACCOUNT_TYPE_ADMINISTRATOR,
                                                            ACT_USER_ACCOUNT_TYPE_STANDARD,
                                                            G_PARAM_READABLE));
         g_object_class_install_property (gobject_class,
@@ -395,6 +400,15 @@ act_user_class_init (ActUserClass *class)
                                                                FALSE,
                                                                G_PARAM_READABLE));
 
+        g_object_class_install_property (gobject_class,
+                                         PROP_SYSTEM_ACCOUNT,
+                                         g_param_spec_boolean ("system-account",
+                                                               "System Account",
+                                                               "System Account",
+                                                               FALSE,
+                                                               G_PARAM_READABLE));
+
+
         signals [CHANGED] =
                 g_signal_new ("changed",
                               G_TYPE_FROM_CLASS (class),
@@ -506,7 +520,12 @@ act_user_get_real_name (ActUser *user)
 {
         g_return_val_if_fail (ACT_IS_USER (user), NULL);
 
-        return (user->real_name ? user->real_name : user->user_name);
+        if (user->real_name == NULL ||
+            user->real_name[0] == '\0') {
+                return user->user_name;
+        }
+
+        return user->real_name;
 }
 
 /**
@@ -767,6 +786,21 @@ act_user_get_automatic_login (ActUser *user)
         return user->automatic_login;
 }
 
+/**
+ * act_user_is_system_account:
+ * @user: a #ActUser
+ *
+ * Returns whether or not #ActUser represents a 'system account' like
+ * 'root' or 'nobody'.
+ *
+ * Returns: %TRUE or %FALSE
+ */
+gboolean
+act_user_is_system_account (ActUser *user)
+{
+        return user->system_account;
+}
+
 /**
  * act_user_get_icon_file:
  * @user: a #ActUser
@@ -965,6 +999,14 @@ collect_props (const gchar    *key,
                         user->automatic_login = new_automatic_login_state;
                         g_object_notify (G_OBJECT (user), "automatic-login");
                 }
+        } else if (strcmp (key, "SystemAccount") == 0) {
+                gboolean new_system_account_state;
+
+                new_system_account_state = g_value_get_boolean (value);
+                if (new_system_account_state != user->system_account) {
+                        user->system_account = new_system_account_state;
+                        g_object_notify (G_OBJECT (user), "system-account");
+                }
         } else if (strcmp (key, "LoginFrequency") == 0) {
                 int new_login_frequency;
 
@@ -1019,6 +1061,7 @@ on_get_all_finished (DBusGProxy     *proxy,
         gboolean     res;
 
         g_assert (user->get_all_call == call);
+        g_assert (DBUS_IS_G_PROXY (user->object_proxy));
         g_assert (user->object_proxy == proxy);
 
         error = NULL;
@@ -1029,13 +1072,12 @@ on_get_all_finished (DBusGProxy     *proxy,
                                      &hash_table,
                                      G_TYPE_INVALID);
         user->get_all_call = NULL;
-        user->object_proxy = NULL;
 
         if (! res) {
                 g_debug ("Error calling GetAll() when retrieving properties for %s: %s",
                          user->object_path, error->message);
                 g_error_free (error);
-                goto out;
+                return;
         }
         g_hash_table_foreach (hash_table, (GHFunc) collect_props, user);
         g_hash_table_unref (hash_table);
@@ -1045,23 +1087,16 @@ on_get_all_finished (DBusGProxy     *proxy,
         }
 
         g_signal_emit (user, signals[CHANGED], 0);
-
-out:
-        g_object_unref (proxy);
 }
 
 static gboolean
 update_info (ActUser *user)
 {
-        DBusGProxy     *proxy;
         DBusGProxyCall *call;
 
-        proxy = dbus_g_proxy_new_for_name (user->connection,
-                                           ACCOUNTS_NAME,
-                                           user->object_path,
-                                           DBUS_INTERFACE_PROPERTIES);
+        g_assert (DBUS_IS_G_PROXY (user->object_proxy));
 
-        call = dbus_g_proxy_begin_call (proxy,
+        call = dbus_g_proxy_begin_call (user->object_proxy,
                                         "GetAll",
                                         (DBusGProxyCallNotify)
                                         on_get_all_finished,
@@ -1076,15 +1111,14 @@ update_info (ActUser *user)
                 goto failed;
         }
 
+        if (user->get_all_call != NULL) {
+                dbus_g_proxy_cancel_call (user->object_proxy, user->get_all_call);
+        }
+
         user->get_all_call = call;
-        user->object_proxy = proxy;
         return TRUE;
 
 failed:
-        if (proxy != NULL) {
-                g_object_unref (proxy);
-        }
-
         return FALSE;
 }
 
@@ -1125,6 +1159,11 @@ _act_user_update_from_object_path (ActUser    *user,
         dbus_g_proxy_connect_signal (user->accounts_proxy, "Changed",
                                      G_CALLBACK (changed_handler), user, NULL);
 
+        user->object_proxy = dbus_g_proxy_new_for_name (user->connection,
+                                                        ACCOUNTS_NAME,
+                                                        user->object_path,
+                                                        DBUS_INTERFACE_PROPERTIES);
+
         if (!update_info (user)) {
                 g_warning ("Couldn't update info for user with object path %s", object_path);
         }
