@@ -65,7 +65,8 @@ enum {
         PROP_LOGIN_FREQUENCY,
         PROP_ICON_FILE,
         PROP_LOCKED,
-        PROP_AUTOMATIC_LOGIN
+        PROP_AUTOMATIC_LOGIN,
+        PROP_SYSTEM_ACCOUNT
 };
 
 enum {
@@ -100,6 +101,7 @@ struct User {
         gchar        *icon_file;
         gboolean      locked;
         gboolean      automatic_login;
+        gboolean      system_account;
 };
 
 typedef struct UserClass
@@ -138,6 +140,9 @@ user_set_property (GObject      *object,
         case PROP_AUTOMATIC_LOGIN:
                 user->automatic_login = g_value_get_boolean (value);
                 break;
+        case PROP_SYSTEM_ACCOUNT:
+                user->system_account = g_value_get_boolean (value);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
                 break;
@@ -208,6 +213,9 @@ user_get_property (GObject    *object,
         case PROP_AUTOMATIC_LOGIN:
                 g_value_set_boolean (value, user->automatic_login);
                 break;
+        case PROP_SYSTEM_ACCOUNT:
+                g_value_set_boolean (value, user->system_account);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
                 break;
@@ -361,6 +369,14 @@ user_class_init (UserClass *class)
                                                                "Automatic Login",
                                                                FALSE,
                                                                G_PARAM_READWRITE));
+
+        g_object_class_install_property (gobject_class,
+                                         PROP_SYSTEM_ACCOUNT,
+                                         g_param_spec_boolean ("system-account",
+                                                               "System Account",
+                                                               "System Account",
+                                                               FALSE,
+                                                               G_PARAM_READWRITE));
 }
 
 
@@ -383,6 +399,7 @@ user_init (User *user)
         user->password_hint = NULL;
         user->locked = FALSE;
         user->automatic_login = FALSE;
+        user->system_account = FALSE;
 }
 
 static void
@@ -409,41 +426,38 @@ user_finalize (GObject *object)
 }
 
 static gint
-account_type_from_groups (struct passwd *pwent)
+account_type_from_pwent (struct passwd *pwent)
 {
         struct group *grp;
-        gid_t desktop_user_r;
-        gid_t desktop_admin_r;
+        gid_t wheel;
         gid_t *groups;
         gint ngroups;
         gint i;
 
-        grp = getgrnam ("desktop_user_r");
-        if (grp == NULL) {
-                g_warning ("desktop_user_r group not found");
-                return ACCOUNT_TYPE_STANDARD;
+        if (pwent->pw_uid == 0) {
+                g_debug ("user is root so account type is administrator");
+                return ACCOUNT_TYPE_ADMINISTRATOR;
         }
-        desktop_user_r = grp->gr_gid;
 
-        grp = getgrnam ("desktop_admin_r");
+        grp = getgrnam ("wheel");
         if (grp == NULL) {
-                g_warning ("desktop_admin_r group not found");
+                g_debug ("wheel group not found");
                 return ACCOUNT_TYPE_STANDARD;
         }
-        desktop_admin_r = grp->gr_gid;
+        wheel = grp->gr_gid;
 
         ngroups = get_user_groups (pwent->pw_name, pwent->pw_gid, &groups);
 
         for (i = 0; i < ngroups; i++) {
-                if (groups[i] == desktop_user_r)
-                        return ACCOUNT_TYPE_STANDARD;
-                if (groups[i] == desktop_admin_r)
+                if (groups[i] == wheel) {
+                        g_free (groups);
                         return ACCOUNT_TYPE_ADMINISTRATOR;
+                }
         }
 
         g_free (groups);
 
-        return ACCOUNT_TYPE_SUPERVISED;
+        return ACCOUNT_TYPE_STANDARD;
 }
 
 void
@@ -514,7 +528,7 @@ user_local_update_from_pwent (User          *user,
         /* GID */
         user->gid = pwent->pw_gid;
 
-        user->account_type = account_type_from_groups (pwent);
+        user->account_type = account_type_from_pwent (pwent);
 
         /* Username */
         if (g_strcmp0 (user->user_name, pwent->pw_name) != 0) {
@@ -577,6 +591,10 @@ user_local_update_from_pwent (User          *user,
                 g_object_notify (G_OBJECT (user), "password-mode");
         }
 
+        user->system_account = daemon_local_user_is_excluded (user->daemon,
+                                                              user->user_name,
+                                                              user->uid);
+
         g_object_thaw_notify (G_OBJECT (user));
 
         if (changed)
@@ -1352,6 +1370,30 @@ user_change_icon_file_authorized_cb (Daemon                *daemon,
 
         filename = g_strdup (data);
 
+        if (filename == NULL ||
+            *filename == '\0') {
+                char *dest_path;
+                GFile *dest;
+                GError *error;
+
+                g_free (filename);
+                filename = NULL;
+
+                dest_path = g_build_filename (ICONDIR, user->user_name, NULL);
+                dest = g_file_new_for_path (dest_path);
+                g_free (dest_path);
+
+                error = NULL;
+                if (!g_file_delete (dest, NULL, &error)) {
+                        g_object_unref (dest);
+                        throw_error (context, ERROR_FAILED, "failed to remove user icon, %s", error->message);
+                        g_error_free (error);
+                        return;
+                }
+                g_object_unref (dest);
+                goto icon_saved;
+        }
+
         file = g_file_new_for_path (filename);
         info = g_file_query_info (file, G_FILE_ATTRIBUTE_UNIX_MODE ","
                                         G_FILE_ATTRIBUTE_STANDARD_SIZE,
@@ -1363,7 +1405,7 @@ user_change_icon_file_authorized_cb (Daemon                *daemon,
         g_object_unref (file);
 
         if (size > 1048576) {
-                g_warning ("file too large\n");
+                g_debug ("file too large\n");
                 /* 1MB ought to be enough for everybody */
                 throw_error (context, ERROR_FAILED, "file '%s' is too large to be used as an icon", filename);
                 g_free (filename);
@@ -1447,6 +1489,7 @@ user_change_icon_file_authorized_cb (Daemon                *daemon,
                 filename = dest_path;
         }
 
+icon_saved:
         g_free (user->icon_file);
         user->icon_file = filename;
 
@@ -1564,8 +1607,7 @@ user_change_account_type_authorized_cb (Daemon                *daemon,
         gid_t *groups;
         gint ngroups;
         GString *str;
-        gid_t desktop_user_r;
-        gid_t desktop_admin_r;
+        gid_t wheel;
         struct group *grp;
         gint i;
         gchar *argv[5];
@@ -1575,34 +1617,24 @@ user_change_account_type_authorized_cb (Daemon                *daemon,
                          "change account type of user '%s' (%d) to %d",
                          user->user_name, user->uid, account_type);
 
-                grp = getgrnam ("desktop_user_r");
+                grp = getgrnam ("wheel");
                 if (grp == NULL) {
-                        throw_error (context, ERROR_FAILED, "failed to set account type: desktop_user_r group not found");
+                        throw_error (context, ERROR_FAILED, "failed to set account type: wheel group not found");
                         return;
                 }
-                desktop_user_r = grp->gr_gid;
-
-                grp = getgrnam ("desktop_admin_r");
-                if (grp == NULL) {
-                        throw_error (context, ERROR_FAILED, "failed to set account type: desktop_admin_r group not found");
-                        return;
-                }
-                desktop_admin_r = grp->gr_gid;
+                wheel = grp->gr_gid;
 
                 ngroups = get_user_groups (user->user_name, user->gid, &groups);
 
                 str = g_string_new ("");
                 for (i = 0; i < ngroups; i++) {
-                        if (groups[i] == desktop_user_r || groups[i] == desktop_admin_r)
+                        if (groups[i] == wheel)
                                 continue;
                         g_string_append_printf (str, "%d,", groups[i]);
                 }
                 switch (account_type) {
-                case ACCOUNT_TYPE_STANDARD:
-                        g_string_append_printf (str, "%d", desktop_user_r);
-                        break;
                 case ACCOUNT_TYPE_ADMINISTRATOR:
-                        g_string_append_printf (str, "%d", desktop_admin_r);
+                        g_string_append_printf (str, "%d", wheel);
                         break;
                 default:
                         /* remove excess comma */
